
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  PSGConf::Action::GenerateFile::RC_Script - action to generate RC scripts
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::RC_Script;

use strict;

use PSGConf::Action::GenerateFile;

use File::Basename;

our @ISA = qw(PSGConf::Action::GenerateFile);

###############################################################################
###  generate method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ( $links, $link, $levels, $start_order, $stop_order, $short_name, $s );

    return 0
      if (!exists( $self->{links} )
        || ref $self->{links} ne 'HASH'
        || !exists( $self->{start} )
        || !exists( $self->{stop} )
        || !exists( $self->{restart} ) );

    $short_name = basename $self->{name}, '.sh';

    foreach $link ( sort keys %{ $self->{links} } )
    {
        $s = $self->{links}->{$link};
        $s =~ s/^[KS]//;
        if ( $self->{links}->{$link} =~ m/^S(\d+)/ )
        {
            $levels .= $link;
            if ( !defined $start_order )
            {
                $start_order = $s;
            } elsif ( $start_order ne $s )
            {
                warn "\t!!!Inconsistant start order\n";
            }
        } elsif ( $self->{links}->{$link} =~ m/^K(\d+)/ )
        {
            if ( !defined $stop_order )
            {
                $stop_order = $s;
            } elsif ( $stop_order ne $s )
            {
                warn "\t!!!Inconsistant stop order\n";
            }
        }
    }

    $self->{restart} = $self->{name} . " stop;\n\t\t" . $self->{name} . ' start'
      if ( !defined $self->{restart} );

    $self->{top_cmds}    = '' if ( !defined $self->{top_cmds} );
    $self->{description} = '' if ( !defined $self->{description} );

    if ( $psgconf->data_obj('platform')
        ->match('(-rhel-)|(-rhl-)|(-fc-)|(-centos-)') )
    {
        print $fh <<EOF;

# chkconfig: $levels $start_order $stop_order
# description: $self->{description}
EOF
    }

    print $fh <<EOF;

_err=0
$self->{top_cmds}

case "\$1" in
	start)
EOF
    if ( $psgconf->data_obj('platform')
        ->match('(-rhel-)|(-rhl-)|(-fc-)|(-centos-)') )
    {
        print $fh <<EOF;
		if [ -f /var/lock/subsys/$short_name ]; then
			echo "lockfile (/var/lock/subsys/$short_name) exists!"
			exit 0;
		fi
EOF
    }

    print $fh <<EOF;
		$self->{rc_echo} "Starting $short_name:\\c";
		$self->{start}
		_err=\$?
		$self->{rc_echo} " done.";
EOF

    if ( $psgconf->data_obj('platform')
        ->match('(-rhel-)|(-rhl-)|(-fc-)|(-centos-)') )
    {
        print $fh <<EOF;
		if [ \$_err -eq 0 ]; then
			touch /var/lock/subsys/$short_name;
		fi
EOF
    }

    print $fh <<EOF;
		;;
	stop)
EOF

    if ( $psgconf->data_obj('platform')
        ->match('(-rhel-)|(-rhl-)|(-fc-)|(-centos-)') )
    {
        print $fh <<EOF;
		if [ ! -f /var/lock/subsys/$short_name ]; then
			echo "no lockfile found!"
			exit 0;
		fi
EOF
    }

    print $fh <<EOF;
		$self->{rc_echo} "Stopping $short_name:\\c";
		$self->{stop}
		_err=\$?
		$self->{rc_echo} " done.";
EOF

    if ( $psgconf->data_obj('platform')
        ->match('(-rhel-)|(-rhl-)|(-fc-)|(-centos-)') )
    {
        print $fh <<EOF;
		if [ \$_err -eq 0 ]; then
			rm -f /var/lock/subsys/$short_name;
		fi
EOF
    }

    print $fh <<EOF;
		;;
	restart)
		$self->{restart};
		;;
	*)
		$self->{rc_echo} "Usage: \$0 {start|stop|restart}";
		exit 1;
		;;
esac

exit \$_err;
EOF

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::RC_Script - action to generate RC scripts

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::RC_Script;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::RC_Script->new(
			'name'			=> '/etc/init.d/scriptname',
			'description'		=> 'What this script does',
			'rc_echo'			=> 'echo command',
			'start'			=> '...',
			'stop'			=> '...',
			'restart'			=> '...',
			'top_cmds'		=> '...',
			'links'			=> '...',
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::RC_Script> module provides a B<PSGConf>
action class to generate an RC script for a particular subsystem.

The B<PSGConf::Action::GenerateFile::RC_Script> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Creates an RC script.  The text specified by the I<top_cmds> attribute
is placed at the top of the script.  The commands specified by the I<start>
and I<stop> and optionally I<restart> attributes are used to start and stop 
the subsystem controlled by the RC script.  If the I<restart> is not defined,
then it will default to calling itself with the I<stop> followed by the 
I<start> arguments.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::RC_Script> class supports the following
attributes:

=over 4

=item I<name>

The file name of the script to write to.

=item I<description>

A description of what the script is for, used for C<chkconfig> options.

=item I<top_cmds>

The commands to run at the top of the script.

=item I<start>

The commands to run to start the subsystem.

=item I<stop>

The commands to run to stop the subsystem.

=item I<restart>

The commands to run to restart the subsystem, if different than the
I<stop> and I<start> commands.

=item I<links>

A B<PSGConf::Data::Hash> to determine what run levels the script is run at.

=item I<rc_echo>

A B<PSGConf::Data::String> to determine what command to use to print
output to the screen.  Must support the \c for no newline option.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut

