
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  etc_master_passwd.pm - psgconf action to generate /etc/master.passwd
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::etc_master_passwd;

use strict;

use PSGConf::Action::GenerateFile;

use PSGConf::Util;

our @ISA = qw(PSGConf::Action::GenerateFile);

###############################################################################
###  constructor
###############################################################################

sub new
{
    my ( $class, %opts ) = @_;

    umask 077;
    $opts{tmpfile} = '/etc/master.passwd.tmp';

    return PSGConf::Action::GenerateFile::new( $class, %opts );
}

###############################################################################
###  generate() method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ( $entry, $sort_func );

    return 0 if ( !exists( $self->{sort_func} ) );

    $sort_func = $self->{sort_func};

    foreach $entry ( sort $sort_func values %{ $self->{user_info} } )
    {
        print $fh $entry->{name} . ':';
        print $fh (
            defined( $entry->{passwd} )
            ? $entry->{passwd}
            : '*'
        ) . ':';
        print $fh $entry->{uid} . ':';
        print $fh $entry->{gid} . ':';
        print $fh $entry->{class} . ':';
        print $fh (
            defined( $entry->{change} )
            ? $entry->{change}
            : '0'
        ) . ':';
        print $fh (
            defined( $entry->{expire} )
            ? $entry->{expire}
            : '0'
        ) . ':';
        print $fh $entry->{gecos} . ':';
        print $fh $entry->{home} . ':';
        print $fh $entry->{shell};
        print $fh "\n";
    }

    return 1;
}

###############################################################################
###  do() method
###############################################################################

sub do
{
    my ( $self, $psgconf ) = @_;
    my ( $cmd, $err );

    $cmd = 'PW_SCAN_BIG_IDS=1 /usr/sbin/pwd_mkdb -p ' . $self->{tmpfile};

    return -1
      if ( &PSGConf::Util::RunCommand($cmd) );

    ### remove tmpfile if applicable
    if (   -e $self->{tmpfile}
        && $psgconf->{rm_tmpfiles}
        && !unlink( $self->{tmpfile} ) )
    {
        warn "\t!!! unlink('$self->{tmpfile}'): $!\n";
    }

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::etc_master_passwd - generate /etc/master.passwd file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::etc_master_passwd;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::etc_master_passwd->new(
			'name'		=> '/etc/master.passwd',
			'user_info'	=> {
				'username'	=> { ... },
				...
			},
			'sort_func'	=> \&_sort_func,
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::etc_master_passwd> module provides
a B<PSGConf> action class for generating the F</etc/master.passwd> file.

The B<PSGConf::Action::GenerateFile::etc_master_passwd> class is derived
from the B<PSGConf::Action::GenerateFile> class, but it defines/overrides
the following methods:

=over 4

=item generate()

Generates the F</etc/master.passwd> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::etc_master_passwd> class supports the
following attributes:

=over 4

=item I<user_info>

A hash mapping user names to a hash containing the fields for that user.
The hash for each user should contain the fields I<name>, which is set
to the name of the user; I<uid>, which is set to the numeric uid of the
user; I<gid>, which is set to the numeric gid of the user's primary
group; I<gecos>, which is set to the user's GECOS information; I<home>,
which is set to the user's home directory; and I<shell>, which is set to
the user's login shell.

=item I<sort_func>

A reference to a subroutine that will be used to sort the entries in
I<user_info>.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

L<PSGConf::Util>

=cut


