
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  etc_shadow.pm - psgconf action to generate /etc/shadow
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::etc_shadow;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);

###############################################################################
###  generate() method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ( $entry, $sort_func );

    return 0
      if ( !exists( $self->{sort_func} ) );

    $sort_func = $self->{sort_func};

    foreach $entry ( sort $sort_func values %{ $self->{user_info} } )
    {
        if ( defined $entry->{name} )
        {
            print $fh $entry->{name} . ':';
            print $fh (
                defined( $entry->{passwd} )
                ? $entry->{passwd}
                : '*'
            );
            print $fh ':';    ### lastchg
            print $fh ':';    ### min
            print $fh ':';    ### max
            print $fh ':';    ### warn
            print $fh ':';    ### inactive
            print $fh ':';    ### expire
            print $fh ':';    ### flag
            print $fh "\n";
        }
    }

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::etc_shadow - generate /etc/shadow file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::etc_shadow;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::etc_shadow->new(
			'name'		=> '/etc/shadow',
			'user_info'	=> {
				'user'	=> {
					'name' => 'user',
					'passwd' => '...',
					...
				},
				...
			},
			'sort_func'	=> \&_sort_func,
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::etc_shadow> module provides a
B<PSGConf> action class for generating the F</etc/shadow> file.

The B<PSGConf::Action::GenerateFile::etc_shadow> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Generates the F</etc/shadow> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::etc_shadow> class supports the following
attributes:

=over 4

=item I<user_info>

A hash mapping user names to a hash containing the fields for that user.
The hash for each user should contain the fields I<name>, which is set
to the name of the user, and I<passwd>, which is set to the encrypted
password for that user.  It may also contain a field called I<uid>,
which contains the numeric uid of the user; if present, this field will
be used to sort the entries in the file.  Any other fields are ignored.

=item I<sort_func>

A reference to a subroutine that will be used to sort the entries in
I<user_info>.

=back

=head1 BUGS

It might be useful to encode the I<lastchg>, I<min>, I<max>, I<warn>,
I<inactive>, I<expire>, and I<flag> fields for each user.

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut


