
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  PSGConf::Action::GenerateFile::hosts_allow - action for /etc/hosts.allow
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::hosts_allow;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);

###############################################################################
###  generate method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ( $entry, $index, @entries, $maxlines );
    $maxlines = defined $self->{maxlines} ? $self->{maxlines} : 80;

    foreach $entry ( @{ $self->{entries} } )
    {
        my ( $loop, $count ) = ( 0, 0 );

        ### print a unique sorted host list
        my (%hosts);
        map { $hosts{$_} = undef; } split( /\s+/, $entry->[1] );

        @entries = sort keys %hosts;

        # no entries, just spit out the daemon and its options
        if ( $#entries == -1 )
        {
            ### print daemon list
            print $fh "\n$entry->[0] :";

            ### add the remaining options
            for ( $index = 2 ; $index < scalar(@$entry) ; $index++ )
            {
                print $fh " \\\n\t: $entry->[$index]";
            }

            print $fh "\n";
        } else
        {

            # other wise, we want to limit the number of entries to an arbitrary
            # number to keep tcp wrappers happy and not complaining about a line
            # that's too long
            #
            # count is the number of times we've spit out $maxlines entries
            # loop is the current number within that $maxlines that we are on
            while ( ( $count * $maxlines ) < $#entries + 1 )
            {

                ### print daemon list
                print $fh "\n$entry->[0] :";

                # print out $maxlines entries
                while ( $loop < $maxlines )
                {
                    print $fh " \\\n\t"
                      . $entries[ $count * $maxlines + $loop ];

                    ++$loop;

                    # if we've hit the last one, stop looping
                    last if ( $count * $maxlines + $loop > $#entries );
                }

                ### add the remaining options
                for ( $index = 2 ; $index < scalar(@$entry) ; $index++ )
                {
                    print $fh " \\\n\t: $entry->[$index]";
                }

                ### add trailing newline
                print $fh "\n";

                ++$count;
                $loop = 0;
            }
        }
    }

    print $fh "\n";

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::hosts_allow - generate /etc/hosts.allow file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::hosts_allow;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::hosts_allow->new(
			'name'		=> '/etc/hosts.allow',
			'entries'	=> [
				[ ... ],
				...
			],
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::hosts_allow> module provides a
B<PSGConf> action class for generating the TCP wrappers F<hosts.allow>
file.

The B<PSGConf::Action::GenerateFile::hosts_allow> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Generates the F<hosts.allow> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::hosts_allow> class supports the following
attributes:

=over 4

=item I<entries>

An array containing the ordered list of entries to be written to the
file.  Each entry is a reference to a list containing the daemons,
hosts, and options for that entry.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut

