
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  PSGConf::Action::GenerateFile::logadm - psgconf Action for generating /etc/logadm.conf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::logadm;

use strict;

use PSGConf::Action::GenerateFile;
use PSGConf::Util;

our @ISA = qw(PSGConf::Action::GenerateFile);

my %SwitchMap = (
    'a' => 'post_command',
    'C' => 'count',
    'c' => 'copy',
    'g' => 'group',
    'm' => 'mode',
    'M' => 'move_command',
    'N' => 'missingok',
    'l' => 'localtime',
    'o' => 'owner',
    'P' => 'timestamp',
    'p' => 'period',
    's' => 'size',
    't' => 'template',
    'z' => 'compress'
);

my %PeriodMap = (
    'daily'   => '1d',
    'monthly' => '1m',
    'weekly'  => '1w',
    'yearly'  => '1y'
);

my %TemplateMap = ( 'dateext' => '$file.%Y%m%d' );

sub _find_field
{
    my ( $self, $psgconf, $field, $file ) = @_;
    my ( $file2, $res );

    $file2 = &PSGConf::Util::_expand_tokens( $psgconf, $file );

    $res = (
        exists $self->{options}->{$file2}->{$field}
        ? $self->{options}->{$file2}->{$field}
        : (
            exists $self->{options}->{$file}->{$field}
            ? $self->{options}->{$file}->{$field}
            : 'N/A'
        )
    );

    # Translate any special tokens
    $res = &PSGConf::Util::_expand_tokens( $psgconf, $res )
      if ( $res =~ m/%{.*}/ );

    # Map the period back to logadm format.
    $res = exists $PeriodMap{$res} ? $PeriodMap{$res} : $res
      if ( $field eq 'period' );

    # Map the dateext back to a template format as well
    $res = exists $TemplateMap{$res} ? $TemplateMap{$res} : $res
      if ( $field eq 'template' );

    # If the fields have spaces or other special characters in them,
    # wrap them in single quotes
    $res = "'$res'" if ( $res =~ /[\s\$\%]/ );

    return $res;
}

###############################################################################
###  generate() method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ( $file, $line, $switch, $opt );

    foreach $file ( @{ $self->{files} } )
    {
        $line = '';

        # logadm(1m) likes the options listed in the file in assending
        # order with case NOT folded, hence the 'sort' before keys.
        foreach $switch ( sort keys %SwitchMap )
        {
            $opt = _find_field( $self, $psgconf, $SwitchMap{$switch}, $file );
            if ( $opt ne 'N/A' )
            {
                $line .= ' ' if ( length $line && $line !~ /\s$/ );
                $line .= '-' . $switch;
                $line .= ' ' . $opt if ( defined $opt );
            }
        }

        # If the entry has a name token, then put the filename at the end.
        $opt = _find_field( $self, $psgconf, 'name', $file );

        $file = &PSGConf::Util::_expand_tokens( $psgconf, $file )
          if ( $file =~ m/%{.*}/ );

        if ( $opt eq 'N/A' )
        {
            print $fh "$file $line\n";
        } else
        {
            print $fh "$opt $line $file\n";
        }
    }

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::logadm - generate /etc/logadm.conf file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::logadm;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::logadm->new(
			'name'	=> '/etc/logadm.conf',
			'uid'	=> 0,
			'options'	=> { ... },
			'count' => 14,
			'files'	=> [ ... ],
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::logadm> module provides a B<PSGConf>
action class for generating the F</etc/logadm.conf> file.

The B<PSGConf::Action::GenerateFile::logadm> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Creates the F</etc/logadm.conf> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::logadm> class supports the following
attributes:

=over 4

=item I<name>

The name of the file to generate, usually F</etc/logadm.conf>.

=item I<uid>

What owner the F<logadm.conf> file should be.

=item I<options>

Any extra options that are required to be put into F<logadm.conf>.

=over 4

=item I<post_command>

If set, the logfile rotation tools should execute this command after the
file is rotated.

=item I<count>

An integer for the number of files to keep.

=item I<copy>

Tells F<logadm> to copy the file istead of moving it.

=item I<group>

What the group members should be for the log file.

=item I<mode>

What the mode should be set for the log file.

=item I<move_command>

What command to execute to do the file rotation instead of
I<mv $file $file.$ext>.

=item I<missingok>

Tells F<logadm> that if the file does not exist, do not complain about it.

=item I<localtime>

Tells F<logadm> to do everything in C<CE<lt>localtime(3)E<gt>>
instead of GMT.

=item I<owner>

Who the owner of the file should be.

=item I<timestamp>

The C<ctime> format of the last time the log file was rotated.

=item I<period>

How often the log files are rotated.  There are several abstracted names:

=item I<name>

If this is set, then it will put this name at the beginning of the line
and the key to these options at the end.  See the C<smf_logs> entry on
Solaris 10 systems for an example.

=over 4

=item I<daily>

Rotate the file every day.  Translated to F<1d>.

=item I<weekly>

Rotate the file every week.  Translated to F<1w>.

=item I<monthly>

Rotate the file every month.  Translated to F<1m>.

=item I<yearly>

Rotate the file every year.  Translated to F<1y>.

=back

=item I<size>

Rotate the log file when it reaches this size.

=item I<template>

What special extension that should be assigned to the file.  The abtracted
value I<dateext> will be translated to F<$file.%Y%m%d>.

=item I<compress>

If this key exists, it will cause the log files to be compressed after
they are rotated. 

=back

=item I<count>

The default number of files we keep.

=item I<files>

The list of files we rotate.

=back

=head1 SEE ALSO

C<CE<lt>logadm(1m)E<gt>>

C<CE<lt>logadm.conf(4)E<gt>>

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut
