
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  mod_jk.pm - Apache/mod_jk config module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Control::Apache::mod_jk;

use strict;

use PSGConf::Action::GenerateFile::workers_properties;
use PSGConf::Data::Boolean;
use PSGConf::Data::List;
use PSGConf::Data::Hash;
use PSGConf::Control::LogFiles qw(_add_logfiles);
use PSGConf::Control::Packages qw(_add_pkgs);

###############################################################################
###  policy methods
###############################################################################

sub _policy_check_apache
{
    my ( $self, $psgconf ) = @_;

    $psgconf->data_obj('www_jk_enable')->set('false')
      if ( $psgconf->data_obj('www_enable')->equals('false') );
}

### canonify logfile path
sub _policy_canonify_logfile
{
    my ( $self, $psgconf ) = @_;

    return
      if ( $psgconf->data_obj('www_jk_enable')->equals('false') );

    $psgconf->data_obj('www_jk_logfile')
      ->prepend( $psgconf->data_obj('www_log_dir')->get() . '/' )
      if ( $psgconf->data_obj('www_jk_logfile')->get() !~ m|^/| );
}

### add Apache config
sub _policy_add_apache_config
{
    my ( $self, $psgconf ) = @_;
    my ( $www_config_dir, $logfile, $loglevel, $pidfile_dir );

    return
      if ( $psgconf->data_obj('www_jk_enable')->equals('false') );

    ### main server config
    $www_config_dir = $psgconf->data_obj('www_config_dir')->get();
    $logfile        = $psgconf->data_obj('www_jk_logfile')->get();
    $loglevel       = $psgconf->data_obj('www_jk_loglevel')->get();
    $pidfile_dir    = $psgconf->data_obj('pidfile_dir')->get();
    $psgconf->data_obj('www_main_server_config')->append( "
<IfModule mod_jk.c>
    JkWorkersFile	$www_config_dir/jk/workers.properties
    JkLogFile		$logfile
    JkLogLevel		$loglevel
    JkShmFile		$pidfile_dir/jk-runtime-status
</IfModule>

" );
}

### add JkMount directives to vhosts
sub _policy_add_vh_config
{
    my ( $self, $psgconf ) = @_;
    my ( $vhost, $config, $mounts, $text, $uri );

    return
      if ( $psgconf->data_obj('www_jk_enable')->equals('false') );

    $mounts = $psgconf->data_obj('www_vh_jk_mounts')->get();
    $config = $psgconf->data_obj('www_vh_config')->get();

    ### add JkMount directives for each vhost
    foreach $vhost ( keys %{ $psgconf->data_obj('www_virtual_hosts')->get() } )
    {
        next
          if ( !exists( $mounts->{$vhost} ) );

        $text = "\n    <IfModule mod_jk.c>\n";
        foreach $uri ( sort keys %{ $mounts->{$vhost} } )
        {
            $text .= "\tJkMount $uri\t";
            $text .= $mounts->{$vhost}->{$uri} . "\n";
        }
        $text .= "    </IfModule>\n";

        $config->{$vhost} .= $text;
    }
}

### add Apache modules
sub _policy_add_apache_modules
{
    my ( $self, $psgconf ) = @_;

    return
      if ( $psgconf->data_obj('www_jk_enable')->equals('false') );

    $psgconf->data_obj('www_modules')->append_row( ['jk'] )
      if ( !$psgconf->data_obj('www_modules')->find_row( { 0 => 'jk' } ) );
}

### validate that www_vh_jk_mounts does not use invalid workers
sub _policy_validate_jk_mounts
{
    my ( $self, $psgconf ) = @_;
    my ( $mounts, $workers );
    my ( $vhost, $vhmounts, $mnt );

    return
      if ( $psgconf->data_obj('www_jk_enable')->equals('false') );

    $mounts  = $psgconf->data_obj('www_vh_jk_mounts')->get();
    $workers = $psgconf->data_obj('www_jk_workers')->get();

    foreach $vhost ( keys %{ $psgconf->data_obj('www_virtual_hosts')->get() } )
    {
        next
          if ( !exists( $mounts->{$vhost} ) );

        $vhmounts = $mounts->{$vhost};

        foreach $mnt ( keys %$vhmounts )
        {
            if ( !exists( $workers->{ $vhmounts->{$mnt} } ) )
            {
                warn '\n\t!!!www_vh_jk_mounts '
                  . 'for unknown worker "'
                  . $vhmounts->{$mnt}
                  . '" - deleting this mount' . "\n";
                delete $vhmounts->{$mnt};
            }
        }
    }
}

### Add our logfile to the list of file that should be regularly rotated.
sub _policy_add_logfiles
{
    my ( $self, $psgconf ) = @_;

    $self->{logfiles_options} = {
        'count'        => $psgconf->data_obj('logfiles_count')->get(),
        'period'       => 'daily',
        'compress'     => 0,
        'post_command' => $psgconf->data_obj('www_apachectl_path')->get()
          . ' graceful > /dev/null 2>&1',
        'template' => 'dateext'
    };

    push @{ $self->{files} }, $psgconf->data_obj('www_jk_logfile')->get();

    _add_logfiles( $self, $psgconf );
}

###############################################################################
###  decide() method
###############################################################################

sub decide
{
    my ( $self, $psgconf ) = @_;
    my ( $uid, $gid );

    return
      if ( $psgconf->data_obj('www_jk_enable')->equals('false') );

    $uid =
      (
        defined $psgconf->data_obj('user_info')
          ->find( $psgconf->data_obj('www_user_name')->get() ) )
      ? $psgconf->data_obj('user_info')
      ->find( $psgconf->data_obj('www_user_name')->get() )->{'uid'}
      : -1;
    $gid =
      (
        defined $psgconf->data_obj('user_info')
          ->find( $psgconf->data_obj('www_user_name')->get() ) )
      ? $psgconf->data_obj('user_info')
      ->find( $psgconf->data_obj('www_user_name')->get() )->{'gid'}
      : -1;

    ### generate workers.properties file
    $psgconf->register_actions(
        PSGConf::Action::GenerateFile::workers_properties->new(
            name => $psgconf->data_obj('www_config_dir')->get()
              . "/jk/workers.properties",
            comment_str => undef,
            workers     => $psgconf->data_obj('www_jk_workers')->get()
        ),
        PSGConf::Action::CreateFile->new(
            name => $psgconf->data_obj('www_jk_logfile')->get(),
            mode => 0644,
            uid  => $uid,
            gid  => $gid
        )
    );
}

###############################################################################
###  Constructor
###############################################################################

sub new
{
    my ( $class, $psgconf ) = @_;
    my ($self);

    $self = {};
    bless( $self, $class );

    ### So that _add_pkgs knows which directives to look at
    $self->{name}     = 'www_jk';
    $self->{enable}   = $self->{name} . '_enable';
    $self->{packages} = $self->{name} . '_packages';

    $psgconf->register_data(
        www_jk_enable    => PSGConf::Data::Boolean->new( value   => 'false' ),
        www_jk_packages  => PSGConf::Data::List->new(),
        www_jk_logfile   => PSGConf::Data::String->new( value    => 'jk.log' ),
        www_jk_loglevel  => PSGConf::Data::String->new( value    => 'info' ),
        www_jk_workers   => PSGConf::Data::Hash->new( value_type => 'ARRAY' ),
        www_vh_jk_mounts => PSGConf::Data::Hash->new( value_type => 'HASH' )
    );

    $psgconf->register_policy(
        $self,
        jk_check_apache       => '_policy_check_apache',
        jk_validate_jk_mounts => '_policy_validate_jk_mounts',
        jk_canonify_logfile   => '_policy_canonify_logfile',
        jk_add_packages       => '_add_pkgs',
        jk_add_apache_modules => '_policy_add_apache_modules',
        jk_add_apache_config  => '_policy_add_apache_config',
        jk_add_vh_config      => '_policy_add_vh_config',
        jk_add_logfiles       => '_policy_add_logfiles'
    );

    return $self;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::Apache::mod_jk - psgconf control class for Apache mod_jk

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::Apache::mod_jk

=head1 DESCRIPTION

The B<PSGConf::Control::Apache::mod_jk> module provides a B<psgconf>
control object for configuring the Apache F<mod_jk> module.
It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<www_jk_enable>

A B<PSGConf::Data::Boolean> value, indicating whether F<mod_jk> should
be configured.  The default is no.

=item I<www_jk_packages>

A B<PSGConf::Data::List> value, listing what packages to install.

=item I<www_jk_logfile>

A B<PSGConf::Data::String> object that contains the path to the F<mod_jk>
log file.  If not an absolute path, the file will be placed in the
directory specified by the I<www_log_dir> data object, which is provided
by the B<PSGConf::Control::Apache> module.  The default is F<jk.log>.

=item I<www_jk_loglevel>

A B<PSGConf::Data::String> object that contains the log level
for F<mod_jk>.  Legal values are I<info>, I<error>, and I<debug>.
The default is I<info>.

=item I<www_jk_workers>

A B<PSGConf::Data::Hash> object containing an entry for each worker.
The hash key is the symbolic name of the worker.  The value is a list
containing the port, host, and type of the worker.  If the host is not
specified, it defaults to I<localhost>.  If the type is not specified,
it defaults to I<ajp13>.

=item I<www_vh_jk_mounts>

A B<PSGConf::Data::Hash> object.  The hash keys are the virtual host
labels.  The values are hashes themselves, with the keys being URIs and
the values being the worker name to map that URI to.

=back

The constructor also registers the following policy methods:

=over 4

=item I<jk_check_apache>

If the I<www_enable> data object (supplied by the
B<PSGConf::Control::Apache> module) is not set, unsets the
I<www_jk_enable> data object.

=item I<jk_validate_jk_mounts>

Removes any entries in I<www_vh_jk_mounts> that point to workers that
are not defined in I<www_jk_workers>.

=item I<jk_canonify_logfile>

If the value of the I<www_jk_logfile> data object is not an absolute
path, prepends the value of the I<www_log_dir> data object (supplied by
the B<PSGConf::Control::Apache> module).

=item I<jk_add_packages>

Adds F<mod_jk> to the list of requested packages.  (This
uses the I<pkg_install_list> config object supplied by
B<PSGConf::Control::Packages>.)

=item I<jk_add_apache_modules>

Adds the F<mod_jk> module to the I<www_modules> data object (provided by
the B<PSGConf::Control::Apache> module).

=item I<jk_add_apache_config>

Adds some reasonable defaults to the I<www_main_server_config> data
object (provided by the B<PSGConf::Control::Apache> module).

=item I<jk_add_vh_config>

Adds C<JkMount> directives for each virtual host to the I<www_vh_config>
data object (provided by the B<PSGConf::Control::Apache> module).

=item I<jk_add_logfiles>

Adds I<www_jk_logfile> to the list of files to rotate.  This uses
the I<_add_logfiles> method supplied by B<PSGConf::Control::Logfiles>.

=back

=item decide()

If I<www_jk_enable> is set, instantiates a
B<PSGConf::Action::GenerateFile::workers_properties> object to create
the F<jk/workers.properties> file (which is placed underneath the
I<www_config_dir> directory).

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile::workers_properties>

L<PSGConf::Control::Apache>

L<PSGConf::Control::LogFiles>

L<PSGConf::Control::Packages>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::List>

L<PSGConf::Data::Hash>

L<psgconf-intro>

=cut

