
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  LogFiles.pm - Log File Rotation configuration module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Control::LogFiles;

use PSGConf::Control::Packages qw(_add_pkgs);
use PSGConf::Data::Boolean;
use PSGConf::Data::Integer;
use PSGConf::Data::List;
use PSGConf::Data::String;

use strict;

###############################################################################
###  Utility function to be called from other Control modules
###############################################################################

require Exporter;
our @ISA         = qw (Exporter);
our %EXPORT_TAGS = ( 'all' => [qw( _add_logfiles)] );
our @EXPORT_OK   = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT      = qw();

sub _add_logfiles
{
    my ( $self, $psgconf ) = @_;

    if (   !exists( $self->{enable} )
        || $self->{enable} eq ''
        || !exists( $self->{files} ) )
    {
        warn
"\t!!!Control::Packages->_add_logfiles(): no enable/files attribute\n";
        return;
    }

    return
      if (
        (
               $psgconf->data_obj( $self->{enable} )->can('equals')
            && $psgconf->data_obj( $self->{enable} )->equals('false')
        )
      );

    map {
        $psgconf->data_obj('logfiles')->add($_);

        # If there are no options set, use the default values
        if ( $psgconf->data_obj('logfiles_options')->exists($_)
            || exists $self->{logfiles_options} )
        {
            $psgconf->data_obj('logfiles_options')
              ->insert( { $_ => $self->{logfiles_options} } );
        }
    } @{ $self->{files} };
}

###############################################################################
###  policy methods
###############################################################################

sub _policy_add_crontab
{
    my ( $self, $psgconf ) = @_;

    return
      if ( $psgconf->data_obj('logfiles_enable')->equals('false')
        || !defined $psgconf->data_obj('logfiles_cmd')->get()
        || !defined $psgconf->data_obj('logfiles_user')->get() );

    $psgconf->data_obj('crontabs')->insert(
        {
            $psgconf->data_obj('logfiles_user')->get() => [
                {
                    hour    => $psgconf->data_obj('logfiles_hour')->get(),
                    minute  => $psgconf->data_obj('logfiles_minute')->get(),
                    command => $psgconf->data_obj('logfiles_cmd')->get()
                }
            ]
        }
    );
}

sub _policy_check_action
{
    my ( $self, $psgconf ) = @_;

    return
      if ( $psgconf->data_obj('logfiles_enable')->equals('false') );

    if ( !defined $psgconf->data_obj('logfiles_action')->get() )
    {
        warn "\t!!!logfiles_action not set, disabling Control::LogFiles\n";
        $psgconf->data_obj('logfiles_enable')->set('false');
    }
}

sub _policy_merge_timestamps
{
    my ( $self, $psgconf ) = @_;
    my ( $line, $options, $obj, $log, @data );
    local *FP;

    return
      if ( $psgconf->data_obj('logfiles_enable')->equals('false')
        || $psgconf->data_obj('logfiles_action')->get() ne
        "GenerateFile::logadm" );

    if ( !open( FP, $psgconf->data_obj('logfiles_config_file')->get() ) )
    {
        warn "\n\t!!! can't open existing logadm conf file (%s)\n",
          $psgconf->data_obj('logfiles_config_file')->get()
          if ( $psgconf->{verbose} );
        return;
    }
    @data = <FP>;
    close FP;

    foreach $line (@data)
    {

        ### Skip this line if we do not have any timestamp info...
        next if ( $line !~ / -P / );

        chomp $line;

        ### Find the log file we are working with
        $log = $line;
        $log =~ s/ .*$//;

        ### If we are not tracking this log file anymore, skip it.
        next if ( !$psgconf->data_obj('logfiles')->exists("^$log") );

        ### Now get the time stamp itself (w/o the single quotes).
        $line =~ s/^.* -P '//;
        $line =~ s/' .*$//;

        ### Validate that we have a good timestamp.
        next
          if (
            $line != m/
			^
			(\S\S\S)			# Day of the week
			\s+
			(\S\S\S)			# Month
			\s+
			(\d?\d)			# Day
			\s+
			(\d\d:\d\d:\d\d)	# Time (HH:MM:SS)
			\s+
			(\d\d\d\d)		# Year
			$
		/
          );

        $options = $psgconf->data_obj('logfiles_options')->find($log);

        if ( defined $options )
        {
            ### If we already have a timestamp in the config, just go on.
            next if ( exists $options->{timestamp} );

            ### Store the timestamp for use later.
            $options->{timestamp} = $line;

            ### Since we do not have an entry in logfiles,
            ### store the value for use later.
        } else
        {
            $psgconf->data_obj('logfiles_options')
              ->insert( { $log => { 'timestamp' => $line } } );
        }
    }
}

###############################################################################
###  decide() method
###############################################################################

sub decide
{
    my ( $self, $psgconf ) = @_;
    my ( $action, $uid );

    return
      if ( $psgconf->data_obj('logfiles_enable')->equals('false') );

    $action =
      'PSGConf::Action::' . $psgconf->data_obj('logfiles_action')->get();

    eval "use $action";
    die "PSGConf::Control::LogFiles: cannot load '$action': $@\n"
      if ($@);

    $uid =
      (
        defined $psgconf->data_obj('user_info')
          ->find( $psgconf->data_obj('logfiles_user')->get() ) )
      ? $psgconf->data_obj('user_info')
      ->find( $psgconf->data_obj('logfiles_user')->get() )->{'uid'}
      : -1;

    $psgconf->register_actions(
        $action->new(
            name    => $psgconf->data_obj('logfiles_config_file')->get(),
            uid     => $uid,
            options => $psgconf->data_obj('logfiles_options')->get(),
            count   => $psgconf->data_obj('logfiles_count')->get(),
            files   => $psgconf->data_obj('logfiles')->get()
        )
    );
}

###############################################################################
###  constructor
###############################################################################

sub new
{
    my ( $class, $psgconf ) = @_;
    my ($self);

    $self = {};
    bless( $self, $class );

    ### So that _add_pkgs knows which directives to look at
    $self->{name}     = 'logfiles';
    $self->{enable}   = $self->{name} . '_enable';
    $self->{packages} = $self->{name} . '_packages';

    $psgconf->register_data(
        logfiles_enable => PSGConf::Data::Boolean->new( 'value' => 'false' ),
        logfiles_count  => PSGConf::Data::Integer->new( 'value' => 14 ),
        logfiles_action => PSGConf::Data::String->new(),
        logfiles_user   => PSGConf::Data::String->new( 'value'  => 'root' ),
        logfiles_cmd    => PSGConf::Data::String->new(),
        logfiles_config_file => PSGConf::Data::String->new(),
        logfiles_hour        => PSGConf::Data::Integer->new( 'value' => 23 ),
        logfiles_minute      => PSGConf::Data::Integer->new( 'value' => 59 ),
        logfiles_options     => PSGConf::Data::Hash->new( value_optional => 1 ),
        logfiles             => PSGConf::Data::List->new( 'unique' => 1 ),
        logfiles_packages    => PSGConf::Data::List->new()
    );

    $psgconf->register_policy(
        $self,
        logfiles_merge_timestamps => '_policy_merge_timestamps',
        logfiles_check_action     => '_policy_check_action',
        logfiles_add_packages     => '_add_pkgs',
        logfiles_add_crontab      => '_policy_add_crontab'
    );

    return $self;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::LogFiles - psgconf control class for Log File Rotation

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::LogFiles

=head1 DESCRIPTION

The B<PSGConf::Control::LogFiles> module provides a B<psgconf> control object
for configuring Log File rotation.  It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<logfiles_enable>

A B<PSGConf::Data::Boolean> object enabling the LogFiles module.

=item I<logfiles_count>

A B<PSGConf::Data::Integer> object defining how many days of logfiles
we should be keeping.  Default is 14.

=item I<logfiles>

A B<PSGConf::Data::List> object defining the list of logfiles we
should be rotating.

=item I<logfiles_packages>

A B<PSGConf::Data::List> object containing the packages to install.

=item I<logfiles_action>

A B<PSGConf::Data::String> object containing the name of the
C<PSGConf::Action> to call.

=item I<logfiles_user>

A B<PSGConf::Data::String> object containing the owner of the
I<logfiles_config_file>.  Defaults to I<root>.

=item I<logfiles_cmd>

A B<PSGConf::Data::String> object containing the name of the command
to put into cron, via C<PSGConf::Control::Crontabs>.

=item I<logfiles_config_file>

A B<PSGConf::Data::String> object containing the name of the config file
that I<logfiles_action> should write its output to.

=item I<logfiles_hour>

A B<PSGConf::Data::Integer> object containing the hour that I<logfiles_cmd>
should be called by cron.  Defaults to I<23>.

=item I<logfiles_minute>

A B<PSGConf::Data::Integer> object containing the minute that I<logfiles_cmd>
should be call by cron.  Defaults to I<59>.

=item I<logfiles_options>

A B<PSGConf::Data::Hash> object containing the special logfile options
each logfile should be called with.  The key of the hash is the filename,
that is also in I<logfiles>, the value is an anomoyous hash.  The keys of
this anomoyous hash is dependent on what I<logfiles_action> implements.
Here are some of the more common implemented keys.

=over 4

=item I<count>

An integer for the number of files to keep.

=item I<compress>

If this key exists, it will cause the log files to be compressed after
they are rotated. 

=item I<period>

How often the log files are rotated.  There are several abstracted names:

=over 4

=item I<daily>

Rotate the file every day.

=item I<weekly>

Rotate the file every week.

=item I<monthly>

Rotate the file every month.

=item I<yearly>

Rotate the file every year.

=back

=item I<post_command>

If set, the logfile rotation tools should execute this command after the
file is rotated.

=item I<template>

What special extension that should be assigned to the file.  The abtracted
value I<dateext> will stand for I<YYYYMMDD>.

=item I<timestamp>

The C<ctime> format of the last time the log file was rotated.

=item I<uid>

The owner of the I<logfiles_config_file>.

=back

See documentation for C<PSGConf::Action...> for more information.

=back

The constructor also registers the following policy method:

=over 4

=item I<logfiles_check_action>

Checks to see that I<logfiles_action> is set if I<logfiles_enable> is
also set, otherwise it will disable I<logfiles_enable>.

=item I<logfiles_add_packages>

Adds the packges listed in the C<logfiles_packages> directive to the
C<pkg_install_list> list to be installed by the C<Packages>
Control module.

=item I<logfiles_merge_timestamps>

For systems that have I<logfiles_action> set to I<GenerateFile::logadm>,
it will add the timestamp value from the existing I<logfiles_config_file>
and put the values into the I<logfiles_options> hash with a subkey of
I<timestamp>.

=item I<logfiles_add_crontab>

Will add I<logfiles_cmd> to I<logfiles_user>'s cron via
C<PSGConf::Control::Crontabs>, if I<logfiles_enable>, I<logfiles_cmd>
and I<logfiles_user> are defined.

=back

=item decide()

If the I<syslog_enable> data object is set, instantiates and registers
action objects, as follows:

=over 4

=item *

Registers new B<PSGConf::Action::...> object, as defined in
I<logfiles_action>, to generate the F<logfiles_config_file>.

=back

=item I<_add_logfiles($self, $psgconf)>

A public method to be used in other B<PSGConf::Control> modules to add
entries into the I<logfiles> and I<logfiles_options> directives.
Uses @{$self->{files}} as the list of files to add to I<logfiles>
and $self->{logfiles_options} to add to I<logfiles_options>.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile::logadm>

L<PSGConf::Action::GenerateFile::logrotate>

L<PSGConf::Action::GenerateFile::newsyslog>

L<PSGConf::Control::Packages>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::Integer>

L<PSGConf::Data::List>

L<PSGConf::Data::String>

L<psgconf-intro>

=cut

